@@ -7,7 +7,7 @@ from datetime import datetime
 import pyfiglet
 from unitgrade import msum
 import unittest
-from unitgrade.framework import UTextResult
+from unitgrade.runners import UTextResult, UTextTestRunner
 import inspect
 import os
 import argparse
@@ -141,7 +141,6 @@ def evaluate_report(report, question=None, qitem=None, passall=False, verbose=Fa
         q.possible = 0
         q.obtained = 0
         # q_ = {} # Gather score in this class.
-        from unitgrade.framework import UTextTestRunner
         UTextResult.q_title_print = q_title_print # Hacky
         UTextResult.show_progress_bar = show_progress_bar # Hacky.
         UTextResult.number = n
@@ -157,12 +156,12 @@ def evaluate_report(report, question=None, qitem=None, passall=False, verbose=Fa
             if (s,msg) in res.successes:
                 detail['status'] = "pass"
             elif (s,msg) in res.failures:
-                details['status'] = 'fail'
+                detail['status'] = 'fail'
             elif (s,msg) in res.errors:
-                details['status'] = 'error'
+                detail['status'] = 'error'
             else:
                 raise Exception("Status not known.")
-            detail['message'] = msg
+            detail = {**detail, **msg}#['message'] = msg
             details[key] = detail
 
         # q_[s._testMethodName] = ("pass", None)
@@ -170,7 +169,6 @@ def evaluate_report(report, question=None, qitem=None, passall=False, verbose=Fa
         #     q_[s._testMethodName] = ("fail", msg)
         # for (s,msg) in res.errors:
         #     q_[s._testMethodName] = ("error", msg)
-
         # res.successes[0]._get_outcome()
 
         possible = res.testsRun
