@@ -22,13 +22,15 @@ mfloor = lambda x: np.floor(x)
 
 
 class Logger(object):
-    def __init__(self, buffer):
+    def __init__(self, buffer, write_to_stdout=True):
         # assert False
         self.terminal = sys.stdout
+        self.write_to_stdout = write_to_stdout
         self.log = buffer
 
     def write(self, message):
-        self.terminal.write(message)
+        if self.write_to_stdout:
+            self.terminal.write(message)
         self.log.write(message)
 
     def flush(self):
